import { NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { HOOK_COMPONENTS } from '@c8y/ngx-components';
import { CollapseModule } from 'ngx-bootstrap/collapse';

import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterModule as NgRouterModule } from '@angular/router';
import { UpgradeModule as NgUpgradeModule } from '@angular/upgrade/static';
import { CoreModule, RouterModule } from '@c8y/ngx-components';
import {
  DashboardUpgradeModule,
  UpgradeModule,
  HybridAppModule,
  UPGRADE_ROUTES
} from '@c8y/ngx-components/upgrade';
import { AssetsNavigatorModule } from '@c8y/ngx-components/assets-navigator';
import { CockpitDashboardModule } from '@c8y/ngx-components/context-dashboard';
import { ReportsModule } from '@c8y/ngx-components/reports';
import { SensorPhoneModule } from '@c8y/ngx-components/sensor-phone';
import { BinaryFileDownloadModule } from '@c8y/ngx-components/binary-file-download';

import { ChartTemplateService } from './chart-template.service';
import { ChartTemplate } from './chart-template.component';
import { ChartTemplateConfig } from './chart-template-config.component';

import * as previewChartTemplate from './preview-chart-template';

@NgModule({
  imports: [
    // Upgrade module must be the first
    UpgradeModule,
    BrowserAnimationsModule,
    RouterModule.forRoot(),
    NgRouterModule.forRoot([...UPGRADE_ROUTES], { enableTracing: false, useHash: true }),
    CoreModule.forRoot(),
    AssetsNavigatorModule,
    ReportsModule,
    NgUpgradeModule,
    DashboardUpgradeModule,
    CockpitDashboardModule,
    SensorPhoneModule,
    BinaryFileDownloadModule,
    CommonModule,
    FormsModule,
    CollapseModule.forRoot()
  ],
  declarations: [
    ChartTemplate,
    ChartTemplateConfig
  ],
  entryComponents: [
    ChartTemplate,
    ChartTemplateConfig
  ],
  providers: [
    ChartTemplateService,
    {
      provide: HOOK_COMPONENTS,
      multi: true,
      useValue: {
        id: 'chartTemplate.widget',
        label: 'ChartTemplate Widget',
        description: 'Can display an ChartTemplate.',
        previewImage: previewChartTemplate.previewImage,
        component: ChartTemplate,
        configComponent: ChartTemplateConfig,
        data: {
          ng1: {
            options: {
              groupsSelectable: true
            }
          }
        }
      }
    }
  ],
  schemas: [
    CUSTOM_ELEMENTS_SCHEMA
  ]
})
export class AppModule extends HybridAppModule {
  constructor(protected upgrade: NgUpgradeModule) {
    super();
  }
}
