import { Component, Input, OnInit, ViewChild, ElementRef } from "@angular/core";

import * as Plot from "@observablehq/plot";

import { ChartTemplateService } from "./chart-template.service";

@Component({
  selector: "chart-template",
  templateUrl: "./chart-template.component.html",
  styleUrls: ["./chart-template.component.css"],
})
export class ChartTemplate implements OnInit {
  @Input() config;
  @ViewChild('sample', { static: true }) protected chart: ElementRef;

  measurements: any[]; 
  events: any[];
  alarms: any[];
  devices: any[];

  private static PAGE_SIZE = 50; // サーバ負荷を考慮し、最小限として下さい
  
  constructor(private chartTemplateService: ChartTemplateService) {}

  ngOnInit(): void {
    (async () => {
      // Measurements データ取得
      const measurementsFilter = ({
        fragmentType: 'c8y_Temperature',
        dateFrom: new Date("2022-01-01"),
        dateTo: new Date("2030-01-01"),
        pageSize: ChartTemplate.PAGE_SIZE,
      });
      this.measurements = await this.chartTemplateService.getMeasurementsArray(this.config.device.id, measurementsFilter);

      // Alarms データ取得
      const alarmsFilter = ({
        type: '',
        dateFrom: new Date("2022-01-01"),
        dateTo: new Date("2030-01-01"),
        pageSize: ChartTemplate.PAGE_SIZE,
      });
      this.alarms = await this.chartTemplateService.getAlarmsArray(this.config.device.id, alarmsFilter);
    
      // Events データ取得
      const eventsFilter = ({
        fragmentType: '',
        dateFrom: new Date("2022-01-01"),
        dateTo: new Date("2030-01-01"),
        pageSize: ChartTemplate.PAGE_SIZE,
      });
      this.events = await this.chartTemplateService.getEventsArray(this.config.device.id, eventsFilter);
      
      // グラフを表示
      this.chart.nativeElement.appendChild(this.createPlot()); 

    })();
  }

  /**
   * Observable Plot ライブラリを使いグラフを描画する
   * 
   * Observable 上でデータ可視化を試行錯誤し、その結果を以下に貼り付ける。貼り付け時の注意点は以下の通り
   * - データの変数(measurements, events, alarms, devices)に this を追加する
   * - 以下はカスタムウィジェットでの表示に必要なため、残しておく
   *    - const plot = 
   *    - return plot;
   * 
   * @returns グラフ描画用のsvg要素
   * 
   */
  createPlot(): any {
    // 以下に Observable で作成したグラフ描画部分を追記してください
    const plot = Plot.plot({
    marks: [
      Plot.line(this.measurements, {x: "time", y: "c8y_Temperature_T_value", stroke: "lightblue"})
    ]
  });
  
    return plot;
  }
  
}
